def repr_(cls, incl=True) -> str:
    classname = cls.__class__.__name__
    dct = cls.__dict__ if hasattr(cls, "__dict__") else cls
    args = ", ".join(
        [f"{k}={repr(v)}" for (k, v) in dct.items()
        if incl is True or (k in incl)])
    return f"{classname}({args})"

from functools import wraps
from time import time

def timeit(f):
    @wraps(f)
    def wrap(*args, **kw):
        ts = time()
        result = f(*args, **kw)
        te = time()
        print("func:%r took: %2.4f sec \n args:[%r, %r]" % \
          (f.__name__, args[0:1], kw, te-ts))
        return result
    return wrap


#!/usr/bin/python
from os.path import realpath
from inspect import stack
from pysh import pype
import os
import re
import sys
import uuid
import shlex
import msgpack
from pathlib import Path
from ast import literal_eval

from pysh.pype import pype, ScriptException, ScriptRun, default_pipe
from pysh.serializer import encode, decode
from pysh.utils import repr_, timeit

PYSH_LINE = "#$"

injector_imports = """import os
import msgpack
from pathlib import Path
from pysh import Pysh, pysher, pysh, encode, decode, BashBlock, timeit
from pysh.pype import pype, ScriptException
"""

block_injector = injector_imports + """
class BlockInjector():
    def __init__(self, pysher, pipe=None):
        self.pysh = pysher
        env = {"PYSH_RUNNING": "1"}
        # keep single pipe context here
        self.pipe = pipe or pype(extra_env=env)
        with open("blocks.dat", "rb") as f:
            blocks, srcsw = msgpack.unpackb(f.read(), object_hook=decode)
        self.blocks = blocks
        assert self.pysh.srcf == __file__ or self.pysh.srcf == Path(
            __file__), f"Injector not run from where it was injected {self.pysh.srcf} X{__file__}"
        self.pysh.blocks = self.blocks
        self.pysh.srcsw = srcsw

        # Reserialize references
        for i, block in enumerate(self.blocks):
            assert self.blocks[i].serialized == True, "Block was saved without serialized flag"
            self.blocks[i].pysh = self.pysh
            blocks[i].pipe = self.pipe
            # Optionally reset

    def runblock(self, blockid, argvarvals):
        try:
            block = self.getblock(blockid)
            if not block:
                raise IndexError("Couldn't find block ID in block injector")
            block.run(pipe=self.pipe, argvarvals=argvarvals)
        except ScriptException as e:
            if block.raise_errors:
                from traceback import format_exc
                from tempfile import NamedTemporaryFile
                with NamedTemporaryFile(mode="w", delete=False) as f:
                    trace_string = format_exc()
                    message = trace_string.replace("<string>", f.name)
                    f.write(self.pysh.srcsw)
                raise SystemExit(message) from e
            else:
                print("!!!ScriptException", e)
        if "@" in block.flags:
            print(block.stdout.decode("UTF-8").strip())
        return block.stdout

    def getblock(self, blockid):
        block = next((b for b in self.blocks if b.blockid == blockid), None)
        return block

    def getblockidx(self, blockid):
        block = self.getblock(blockid)
        index = self.blocks.index(block)
        return index
block_injector = BlockInjector(pysher=Pysh(__file__))
"""

class BlockInjector():
    def __init__(self, pysher, pipe=None):
        self.pysh = pysher
        env = {"PYSH_RUNNING": "1"}
        # keep single pipe context here
        self.pipe = pipe or pype(extra_env=env)
        with open("blocks.dat", "rb") as f:
            blocks, srcsw = msgpack.unpackb(f.read(), object_hook=decode)
        self.blocks = blocks
        assert self.pysh.srcf == __file__ or self.pysh.srcf == Path(
            __file__), f"Injector not run from where it was injected {self.pysh.srcf} X{__file__}"
        self.pysh.blocks = self.blocks
        self.pysh.srcsw = srcsw

        # Reserialize references
        for i, block in enumerate(self.blocks):
            assert self.blocks[i].serialized == True, "Block was saved without serialized flag"
            self.blocks[i].pysh = self.pysh
            blocks[i].pipe = self.pipe
            # Optionally reset

    def runblock(self, blockid, argvarvals):
        try:
            block = self.getblock(blockid)
            if not block:
                raise IndexError("Couldn't find block ID in block injector")
            block.run(pipe=self.pipe, argvarvals=argvarvals)
        except ScriptException as e:
            if block.raise_errors:
                from traceback import format_exc
                from tempfile import NamedTemporaryFile
                with NamedTemporaryFile(mode="w", delete=False) as f:
                    trace_string = format_exc()
                    message = trace_string.replace("<string>", f.name)
                    f.write(self.pysh.srcsw)
                raise SystemExit(message) from e
            else:
                print("!!!ScriptException", e)
        if "@" in block.flags:
            print(block.stdout.decode("UTF-8").strip())
        return block.stdout

    def getblock(self, blockid):
        block = next((b for b in self.blocks if b.blockid == blockid), None)
        return block

    def getblockidx(self, blockid):
        block = self.getblock(blockid)
        index = self.blocks.index(block)
        return index


class BashBlock(ScriptRun):
    def __init__(self, srcs: str,
                 blockindex: int,
                 position: tuple,
                 srcf: str | Path,
                 shell="bash",
                 pysh=None,
                 pipe=default_pipe,
                 serialized=False,
                 blockid=None,
                 env=None,
                 matches=None,
                 argvarmatches=None,
                 flags=None,
                 **scriptrun_args):
        self.blockid = blockid or str(uuid.uuid4())
        self.pysh = pysh
        self.srcf = srcf
        self.blockindex = blockindex
        self.position = position
        self.serialized = serialized
        self.flags = flags
        env = env or {}
        env = env | {"PYSH_BLOCK": self.blockid}
        self.matches = matches
        self.argvarmatches = argvarmatches
        super().__init__(srcs=srcs, shell=shell, pipe=pipe, env=env, **scriptrun_args)

    @property
    def linecount(self):
        return len(self.srcs.split("\n"))

    def run(self, *args, argvarvals=[], **kwargs):
        srcsw = self.srcs
        # Interpolate the {$$} vars into the source
        if argvarvals or self.argvarmatches:
            assert len(argvarvals) == len(
                self.argvarmatches), "Provided runvars don't match block.argvars"
            for i, v in enumerate(self.argvarmatches):
                srcsw = srcsw.replace(v["total"], argvarvals[i])
        # print("Running block", self.blockid, self.srcf)
        super().run(srcs=srcsw, *args, **kwargs)
        # print("stdout = ", self.stdout.decode("UTF-8"))

    def runs(self, *args, **kwargs):
        self.run(*args, **kwargs)
        return self.stdout

    def runp(self, *args, **kwargs):
        self.run(*args, **kwargs)
        sname = os.path.basename(self.pysh.srcf)
        stdout = self.stdout.decode("UTF-8").strip()
        print(f"[root@pysh {sname} {self.blockindex}]$ {stdout}")
        return self

    def serialize(self):
        return msgpack.packb(self, default=encode)

    def __repr__(self) -> str:
        classname = self.__class__.__name__
        args = ", ".join(
            [f"{k}={repr(v)}" for (k, v) in self.__dict__.items()])
        return f"{classname}({args})"


class Pysh():
    def __init__(self, srcf, init=True, test_mode=False) -> None:
        """
        """
        self.blocks = []
        self.test_mode = test_mode
        self.srcsw = None
        self.pyshed_block = None
        if init is not False:
            self.updatesrc(srcf)

    def readsrc(self):
        srcs = None
        with open(self.srcf, "r") as f:
            srcs = f.read()
        return srcs

    def updatesrc(self, srcf):
        self.srcf = os.path.realpath(srcf)
        self.srcs = self.readsrc()
        self.srclines = self.srcs.split("\n")
        self.blocks = self.findblocks()

    def pysh(self, srcf=None, exits=True, test_mode=None):
        if test_mode:
            self.test_mode = test_mode
        if os.environ.get("PYSH_ROOT", None):
            if self.test_mode:
                print("#####PYSH_ROOT can't run Pysh.pysh() while pysh is active")
            return
        srcf = srcf or self.srcf
        self.updatesrc(srcf)
        ret = self.shyp(exits)
        return self

    def shyp(self, exits=True):
        if os.environ.get("PYSH_ROOT", None):
            if self.test_mode:
                print("#####PYSH_ROOT can't run Pysh.shyp() while pysh is active")
            return
        # Run this script with the wrapped pysh calls and then exit
        root_script, block_file = self.wrapped()
        env = {"PYSH_ROOT": "1"}
        pyshed = BashBlock(root_script,
                           0,
                           (0, -1),
                           self.srcf,
                           shell="python",
                           pysh=self,
                           stdin_pipe=sys.stdin,
                           stderr_pipe=sys.stderr,
                           stdout_pipe=sys.stdout,
                           raise_errors=False,
                           env=env)
        self.pyshed_block = pyshed
        if self.test_mode:
            print("#####PYSH_SHIPPED")
        pyshed.run()
        # print("Pyshed stdout:", pyshed.stdout) # Should be none as piped to sys.stdout
        Path(block_file).unlink()
        if exits:
            sys.exit(pyshed.returncode)

    def wrapped(self, srcs=None, blocks=None):
        srcs = srcs or self.srcs
        blocks = blocks or self.blocks
        srcsw, block_file = self.wrap_imports(srcs, blocks)
        self.srcsw = srcsw
        return (srcsw, block_file)

    def wrap_imports(self, srcs=None, blocks=None):
        srcs = srcs or self.srcs
        blocks = blocks or self.blocks

        # TODO: remove anything before the shyp() call as it will be rerun ??

        # Inject pysh header
        srclines = srcs.split("\n")
        file_injector = f"__file__ = '{self.srcf}'\n"
        header = f"{file_injector}{block_injector}\n"
        header_length = len(header)
        srcsw = header + '\n'.join(srclines)

        # TODO additional modes, assignment returns besides stdout
        next_start = None
        difference = 0
        for i, block in enumerate(blocks):
            _, spanend = block.position
            start, end = spanend
            slength = (end - start)
            block_start_m = next(
                (m for m in block.matches if "linecontents" in m and m["linecontents"]))
            if len(block_start_m["mode"]) > 1:
                extern_block = True
            length = sum(
                [len(m.get("line", "") or "")
                 for m in block.matches
                 if "linecontents" in m])
            first_match = next((m for m in block.matches if not m["pyvar"]))
            sindex = srcsw.index(first_match["block"])
            assert slength == length, f"mismatch {slength} {length}"
            args = ",".join([match['argname']
                            for match in block.argvarmatches])
            runblock = f"block_injector.runblock('{block.blockid}', argvarvals=[{args}])"
            next_start = sindex  # header_length + start_accum + start - difference
            eend = next_start+length-1
            # difference = (eend - next_start)
            # assert difference == length - 1, f"diff mismatch {difference} {length}"
            srcsw = srcsw[:next_start] + runblock + srcsw[eend:]
            # print("S2", sindex, next_start, difference, header_length, length, start_accum)

        # replace assignment quotes
        srcsw = srcsw.replace('""block_injector', "block_injector")

        # Save blocks to serial file
        block_file = "blocks.dat"
        with open(block_file, "wb") as f:
            f.write(msgpack.packb(([block
                    for block in self.blocks], srcsw), default=encode))

        # print("#----#")
        # print(srcsw)
        # print("#----#")

        return (srcsw, block_file)

    def findblocks(self, srcs=None):
        if os.environ.get("PYSH_ROOT", None):
            if self.test_mode:
                print("#####PYSH_ROOT can't run Pysh.findblocks() while pysh is active")
            return
        if srcs:
            self.blocks = []
        srcs = srcs or self.srcs
        if self.blocks:
            return self.blocks

        # Extract the pysh syntax lines with regex
        pattern = r"(?P<block>(?P<pyvar>.*)(?P<assign>\=\s\"{2})|(?P<line>(?P<command>(?P<init>(?<!\#)\#)(?P<mode>\$+)(?P<flags>(%{0,1})(@{0,1})(!{0,1}))(?P<shell>\w*)?)(?P<space>[\s]+)(?P<linecontents>.*)(?P<eol>\n)))"
        matches = re.finditer(pattern, srcs)
        assert matches, "Root source file doesn't contain any Pysh"
        matches = list(matches)

        # Group the matches into blocks
        accum = -1
        mblock_i = 0
        match_blocks = []
        for i, match in enumerate(matches):
            if not match["mode"]:
                assert match["pyvar"], "Pyvar line can't have mode"
            if not match["pyvar"]:
                assert match["mode"], "Mode line can't have pyvar"

            lstart, lend = match.span()
            if mblock_i not in match_blocks:
                match_blocks.append({"matches": []})
            match_blocks[mblock_i]["matches"].append(match)
            next_match = matches[i+1] if not i > len(matches)-2 else None
            if next_match:
                nlstart, nlend = next_match.span()
                next_match_groups = next_match.groupdict()
                if (lend != nlstart  # Doesn't immediately end
                    # Next line is a pyvar, this shouldnt really happen
                    or not next_match_groups["mode"]
                    # Pyvar should always be in next group
                    or (next_match_groups["pyvar"])
                        or (len(next_match_groups["mode"]) > 1 and not match["pyvar"])):  # Next line is extern, and this line is not assignment
                    mblock_i += 1
                    continue
            accum += 1
        match_blocks = match_blocks[0:-accum]

        # Create BashBlock objects from the block-grouped matches
        def reduce_mblocks_t(mblock):
            return "".join([str((match.groupdict()["linecontents"])) + str((match.groupdict()["eol"]))
                            for match in mblock["matches"]
                            if match.groupdict()["linecontents"] is not None])  # [:-1] # Ignore pre-pysh assignment, trim last \n
        new_blocks = []
        for i, mblock in enumerate(match_blocks):
            matches = mblock["matches"]
            sstart, send = matches[0].span()
            estart, eend = matches[-1].span()
            whole = (sstart, eend)
            matches_groups = sorted(
                [m.groupdict() | {"_span": m.span()} for m in matches], key=lambda x: x["_span"][0])
            assignment = next(
                (mgroup for mgroup in matches_groups if mgroup["assign"]), None)
            block_start_m = next(
                (m for m in matches_groups if "linecontents" in m and m["linecontents"]))
            shell = next((mgroup for mgroup in matches_groups if mgroup["shell"] and not mgroup["assign"]), {
                         "shell": None})["shell"]
            spanend = (assignment["_span"][1], eend) if assignment else whole
            position = (whole, spanend)
            block_srcs = reduce_mblocks_t(mblock)
            pattern2 = r"(?P<total>({\$(?P<argname>[\w_]+)\$}))"
            argvarmatches = re.finditer(pattern2, block_srcs)
            argvarmatches = list(argvarmatches)
            argvarmatches = [
                m.groupdict() | {"_span": m.span()} for m in argvarmatches]
            for m in matches_groups:
                assert m["block"] == m["line"] if m[
                    "line"] is not None else True, f"line doesnt match block \n{m['block']}\n{m['line']}"
            command_args = []
            raw_command = False
            if len(block_start_m["mode"] or "") > 1:
                assert len(matches_groups) == 1 or (matches_groups[0]["pyvar"] and len(matches_groups) == 2), \
                    f"Extern blocklinegroup mixed with others \n {matches_groups}"
                command_args = shlex.split(block_start_m["linecontents"])
                raw_command = True
            raise_errors = False
            if "%" in block_start_m["flags"]:
                raise_errors = True
            script_block = BashBlock(srcs=block_srcs,
                                     shell=shell,
                                     blockindex=i,
                                     position=position,
                                     srcf="#internal",
                                     pysh=self,
                                     matches=matches_groups,
                                     argvarmatches=argvarmatches,
                                     command_args=command_args,
                                     raise_errors=raise_errors,
                                     raw_command=raw_command,
                                     flags=block_start_m["flags"])
            new_blocks.append(script_block)
        if self.test_mode:
            for block in new_blocks:
                print(repr_(block, incl=["position", "srcs", "raise_errors"]))
                for match in block.matches:
                    print(repr_(match, incl=["_span", "block", "line"]))
            # sys.exit(0)

        # Assign to self if not running on an arbitrary script source
        if not srcs:
            self.blocks = new_blocks

        return new_blocks

    def __repr__(self) -> str:
        return repr_(self, ["blocks"])


if os.environ.get("PYSH_ROOT", None) != "1":
    # print("#####PYSH_ROOT no creating importable instance")

    # Function wrapper to run on call source
    def x():
        return realpath(stack()[-1].filename)
    pysher = Pysh(x(), init=False)
    pysh = pysher.pysh
else:
    pysher = None
    pysh = None



import lzma
from streamlit.runtime.uploaded_file_manager import UploadedFile, UploadedFileRec
from typing import Any, Optional
import pysh


def decode(obj: Any, force_type: Optional[str] = None) -> Any:
    try:
        obj = lzma.decompress(obj)
    except:
        pass

    if "__serialized_type__" in obj:
        type = obj["__serialized_type__"]

        if type == "BashBlock" or force_type == "BashBlock":
            obj = pysh.BashBlock(**obj["class_dict"])
    return obj


def type_check(obj, class_ref, class_str, force_type=None):
    """
    isinstance and is class are failing across threads when they shouldn't.
    Possibly to do with PYTHONHASHSEED or import hierarchy.

    obj.__class__.__name__ seems to work best
    """
    # print(obj.__dict__)
    # print(vars(obj))
    # print(obj.__class__)
    # print(obj.__class__.__name__)
    # print( obj.__name__,)

    return isinstance(obj, class_ref) \
        or type(obj) is type(class_ref) \
        or type(obj) is class_ref \
        or (hasattr(obj, "__class__") and obj.__class__ is class_ref) \
        or (hasattr(obj, "__class__") and hasattr(obj.__class__, "__name__") and obj.__class__.__name__ == class_str) \
        or force_type == class_str


def encode(obj: Any, force_type: Optional[str] = None) -> Any:
    serialized = None

    if type_check(obj, pysh.BashBlock, "BashBlock", force_type):
        attr_dict = obj.__dict__.copy()
        del attr_dict["pysh"]
        del attr_dict["pipe"]
        attr_dict["serialized"] = True
        serialized = {
            "__serialized_type__": "BashBlock",
            "class_dict": attr_dict
        }
        return serialized

    return serialized if serialized else obj



# py process executor
import os
import subprocess
from pathlib import Path
from tempfile import NamedTemporaryFile
from pysh.utils import repr_, timeit

shells = ("bash", "python")


class pype():
    def __init__(self, pipe_kwargs=None, base_env=None, extra_env=None):
        self.pipe_kwargs = pipe_kwargs
        self.default_shell = shells[0]
        self.base_env = base_env or os.environ.copy()
        self.extra_env = extra_env or {}

    def run_script(self, script: str | Path,
                   shell=None,
                   pipe_kwargs: dict = None,
                   stdout=None,
                   stderr=None,
                   stdin=None,
                   env=None,
                   raise_errors=True,
                   timeout=None,
                   raw_command=False,
                   command_args=[]):
        """ Run string source script in bash or python by passing a command_string string with the -c flag or calling a script file

            :param script: as str to pass a command string to the shell,
                        or as Path object to call a script file

            :returns: (stdout, stderr)
            :raises: error on non-zero process exit code
        """
        env = env or {}
        env = self.base_env | env | self.extra_env
        shell = shell or self.default_shell

        if type(script) == type(Path()):
            command = [shell, str(Path)]
        else:
            command = [shell, "-c", script]
        if raw_command == True:
            command = [shell]
        default_args = ("script", "shell", "pipe_kwargs",
                        "stdin", "raise_errors")
        command_args = [
            a for a in command_args if a not in default_args]  # set -ex
        pipe_kwargs = pipe_kwargs or self.pipe_kwargs or {}
        stdout = stdout or subprocess.PIPE
        stderr = stderr or subprocess.PIPE
        pipe_args = dict(stdout=stdout,
                         stderr=stderr,
                         stdin=stdin or subprocess.PIPE,
                         bufsize=8192,
                         shell=False,
                         env=env,
                         close_fds=True) | pipe_kwargs
        #cwd, text
        proc = subprocess.Popen(command + command_args, **pipe_args)
        try:
            stdout, stderr = proc.communicate(input=stdin, timeout=timeout)
        except subprocess.TimeoutExpired:
            proc.kill()
            stdout, stderr = proc.communicate()
        if raise_errors is None:
            raise_errors = True
        # print("ProcDebug:", raise_errors, proc.returncode)
        if raise_errors and proc.returncode:
            exc_kwargs = dict(stdout=stdout, stderr=stderr,
                              shell=shell, srcs=script)
            raise ScriptException(proc, **exc_kwargs)
        return stdout, stderr, proc.returncode, proc.pid


default_pipe = pype()


class ScriptRun():
    def __init__(self, srcs: str,
                 shell: str = shells[0],
                 returncode: int | None = None,
                 stdout: str | None = None,
                 stderr: str | None = None,
                 hasrun=False,
                 pipe=None,
                 env=None,
                 timeout=None,
                 raise_errors=None,
                 command_args=[],
                 raw_command=False,
                 **kwargs):
        self.pipe = pipe or default_pipe
        self.shell = shell
        self.srcs = srcs
        self.returncode = returncode
        self.stdout = stdout
        self.stderr = stderr
        self.hasrun = hasrun
        self.exectime = None  # TODO
        self.env = env
        self.timeout = timeout
        self.raise_errors = raise_errors
        self.command_args = command_args
        self.raw_command = raw_command
        self.stdout_pipe = kwargs.get("stdout_pipe", None)
        self.stderr_pipe = kwargs.get("stderr_pipe", None)
        self.stdin_pipe = kwargs.get("stdin_pipe", None)

    def run(self, srcs=None, pipe=None, env=None):
        pipe = pipe or self.pipe
        srcs = srcs or self.srcs
        env = env or self.env
        stdout, stderr, returncode, pid = \
            pipe.run_script(srcs,
                            shell=self.shell,
                            stdin=self.stdin_pipe,
                            stdout=self.stdout_pipe,
                            stderr=self.stderr_pipe,
                            raise_errors=self.raise_errors,
                            command_args=self.command_args,
                            raw_command=self.raw_command,
                            env=env,
                            timeout=self.timeout)
        self.returncode = returncode
        self.stdout = stdout
        self.stderr = stderr
        self.pid = pid
        self.hasrun = True
        return returncode


class ScriptException(ScriptRun, Exception):
    def __init__(self, proc, **ScriptRunI):
        self.proc = proc
        super().__init__(**ScriptRunI)
        # For debugging
        with NamedTemporaryFile(mode="w", delete=False) as f:
            f.write(ScriptRunI["srcs"])
            trace = ScriptRunI["stderr"].decode("UTF-8")
            trace = trace.replace("<string>", f.name)
            message = f"Error running {ScriptRunI['shell']} script {f.name} Trace:\n\t{trace}".strip()
            self.message = message
            super(Exception, self).__init__(message)

    def __repr__(self):
        return repr_(self)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pysh']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pyshpype',
    'version': '0.1.2',
    'description': 'Run shell in python',
    'long_description': '## Pysh/Pype\n\nPython source file preprocessor/interpreter and subprocess pipe manager to enable running in-line bash/shell code during python runtime.\n\n```Python\n#!/usr/bin/env python\nfrom pysh import pysh\npysh()\n\n#$@ echo "Pysh activated"\nstdout = ""#$ echo "This is standard out"\nprint(stdout)\n\n##@!python print("Python through a pysh pype")\n```\n\nUse #$ flagged directives to signify shell code to be piped through a subprocess.\n\nWhen you run pysh() execution stops, and the source file will be processed with regex to extract and replace the code blocks with a subprocess wrapper, and then the source file itself is run through it.\n\n##### Real usage\n```Python\n# Script your system in parallel with your python code execution\n# Do anything you can in bash e.g.\n\nbuild_service()\n#$ cd ~/hosted/myservice && docker compose up\n\naggregate_assets()\ncrf = "23"\nin_file = "/path/in.mp4"\nout_file = "/path/out.mp4"\nfmpg_result = ""#$ ffmpeg -i {$in_file$} \\\n#$ -crf {$crf$} -o {$out_file$} \\\n#$ && rm {$in_file$}\nprocess_assets(process_fmpg_stdout(fmpg_result))\n\nprint("Process complete")\n```\n\n### Installation\nFrom PyPI:\n\n`pip3 install pyshpype`\n\nGit to local folder:\n\n`pip3 install -e "git+https://github.com/blipk/pysh.git#egg=pysh"`\n\n\n\n###### General syntax\n```Python\n#!/usr/bin/python\nfrom pysh import pysh\npysh()\n#$ echo "pysh activated" >> .pysh\n\n# Use the @ flag to always print(stdout) to main sys.stdout\n#$@ echo "hello from bash"\n\n# This is a python comment\n#$@ ls .            # shell eol comment\n##$ sudo rm -rf /   # disable pysh line with pysh comment\n\n# Capture stdout from the shell subprocess\nstout = ""#$ echo "I\'m actually a bytes string"\nprint(stdout.decode("UTF-8"))\n\n# Pass any python variable thats in scope to the pysh script\nmy_var = "hello"\nstdout = ""#$ echo "{$myvar$}"\n\n# run external script with double $\n#$$ my_script.sh\n\n# optionally pass arguments to it\n#$$ argumentative_script.sh arg1 arg2\n\n# Use the ! flag to change the shell that interprets the script\n# must support -c command_strings or filepath if external $$\n#$!sh echo "simple"\n#$!perl oysters.pl\n#$$@!bash printscript.sh\n\n# Multiple flags/features\nstdout = ""#$@!python import time\n#$ print("The time and date", time.asctime())\n\n# Use the % flag to catch errors,\n# otherwise they will be printed but not raised\ntry:\n    result = ""#$$% tests/dinger/notfoundscript.sh "argone"\nexcept SystemExit as e:\n    print("Error", e)\n\nif __name__ == "main":\n    print("Before the above script blocks are run")\n```\n\n\n###### Multiple inline pysh\n```Python\n# Pysh runs code in blocks that are executed in-place\n\n# Block 0\n#$ cd $HOME\n\nstdout_block1 = ""#$ echo "first block is multiline"\n#$ echo "line1"\n#$ echo "line2"\n\n# The last script block won\'t be run\nsys.exit(1)\nstdout_block2 == ""#$ echo "Second"\n#$ echo "Block"\n```\n\n\n##### Advanced usage\n```Python\n# run pysh manually\nfrom pysh import Pysh\nsource_file = __file__\npysher = Pysh(source_file)\nblocks = pysher.findblocks()\n\n# Run a a single block\nblocks[0].run()  # Not run in-place, no stdout. Silent.\n\n# Run script block again, and print stdout with label for block\nblocks[0].runp()\n\n# Run all wanted blocks sequentially at this point,\n# and print their stdout with labels\nrun_blocks = [block.runp() for block in blocks\n              if "/root" in block.srcs]\n\n# Start the python interpreter with pysh on source_file\n# This is the same as running pysh(__file__)\npysher.shyp()\n#$ echo "pysh enabled"\n\n# Switch to another source file and run it through pysh\npysher.pysh(__file__)\n\n# Equivalent to above\npysher.updatesrc(__file__)\npysher.pysh()\n\n# Get information about the script blocks at runtime\nt_block = pysher.blocks[0]\nprint(t_block.hasrun, t_block.returncode)\nprint(t_block.srcs, "\\n--\\n", t_block.stdout)\n```',
    'author': 'Blipk A.D.',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/blipk/pysh',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10',
}


setup(**setup_kwargs)
